/**
 * @file	cf_log.h
 * @author	myusgun <myusgun@gmail.com>
 * @version	0.1
 *
 * @example log.c
 */
#ifndef __CF_LOG_H__
#define __CF_LOG_H__

#include "cf_base.h"

#define CF_ERROR_LOG_INITIALIZE			CF_ERROR_LOG - 1
#define CF_ERROR_LOG_INVALID_CTX		CF_ERROR_LOG - 2
#define CF_ERROR_LOG_SET_MULTITHREAD	CF_ERROR_LOG - 3
#define CF_ERROR_LOG_UNSET_MULTITHREAD	CF_ERROR_LOG - 4
#define CF_ERROR_LOG_LOCK_CTX			CF_ERROR_LOG - 5
#define CF_ERROR_LOG_UNLOCK_CTX			CF_ERROR_LOG - 6
#define CF_ERROR_LOG_FLUSH				CF_ERROR_LOG - 7
#define CF_ERROR_LOG_INVALID_MAPID		CF_ERROR_LOG - 8
#define CF_ERROR_LOG_NOT_MAPPED_ID		CF_ERROR_LOG - 9
#define CF_ERROR_LOG_ALREADY_MAPPED_ID	CF_ERROR_LOG - 10
#define CF_ERROR_LOG_NOT_INITIALIZE		CF_ERROR_LOG - 11

/**
 * 로그의 버퍼 크기를 기본 값으로 설정
 *
 * @see CF_Log_CreateCtx, CF_LOG_OPEN
 */
#define CF_LOG_BUFFER_DEFAULT			-1

/**
 * 로그 기록 시 버퍼를 사용하지 않고 즉시 기록
 *
 * @see CF_Log_CreateCtx, CF_LOG_OPEN, CF_Log_Initialize
 */
#define CF_LOG_BUFFER_NO				0

/**
 * 로그에 아이디 넘버를 부여하여 생성 <br />
 * 로그 기록 시, 아이디 넘버를 사용하면 해당 로그로 기록할 수 있음
 *
 * @param __id		부여할 아이디 넘버
 * @param __file	로그 파일 경로
 * @param __memsize	로그 버퍼 크기
 *
 * @see CF_Log_MapCtxID
 */
#define CF_LOG_OPEN(__id,__file,__memsize)				\
	CF_Log_MapCtxID (__id, CF_Log_CreateCtx (__file, __memsize))

/**
 * 아이디 넘버에 해당하는 로그에 쓰기
 *
 * @param __id	로그의 아이디 넘버
 * @param __pf	로그의 프리픽스 문자열
 * @param __fmt	포맷 스트링
 * @param ...	가변 인자
 *
 * @see CF_Log_GetMappedCtx
 */
#define CF_LOG_WRITE(__id,__pf,__fmt,...)				\
	CF_Log_Write (CF_Log_GetMappedCtx (__id),__pf,__fmt,##__VA_ARGS__)

/**
 * 아이디 넘버에 해당하는 로그를 닫고 해제
 *
 * @param __id 로그의 아이디 넘버
 *
 * @see CF_Log_UnmapCtxID
 */
#define CF_LOG_CLOSE(__id)		CF_Log_UnmapCtxID (__id)

/** 로그 컨텍스트 */
typedef void *	CF_Log_Ctx;

#ifdef __cplusplus
extern "C" {
#endif

CF_EXPORT int
CF_Log_Initialize		(const int logPool);

CF_EXPORT int
CF_Log_Finalize			(void);

CF_EXPORT CF_Log_Ctx
CF_Log_CreateCtx		(const char	* path,
						 const int	memsize);

CF_EXPORT int
CF_Log_DestroyCtx		(CF_Log_Ctx ctx);

CF_EXPORT int
CF_Log_SetMultiThread	(CF_Log_Ctx ctx);

CF_EXPORT int
CF_Log_UnsetMultiThread	(CF_Log_Ctx ctx);

CF_EXPORT int
CF_Log_Write			(CF_Log_Ctx	ctx,
						 const char	* prefix,
						 const char	* fmt, ...);

CF_EXPORT int
CF_Log_Flush			(CF_Log_Ctx ctx);

CF_EXPORT int
CF_Log_MapCtxID			(const int			mapid,
						 const CF_Log_Ctx	ctx);

CF_EXPORT int
CF_Log_UnmapCtxID		(const int mapid);

CF_EXPORT CF_Log_Ctx
CF_Log_GetMappedCtx		(const int mapid);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __CF_LOG_H__
